#!/usr/bin/env python
#
# Copyright 2015-2016 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re

# from http://detectmobilebrowsers.com/
reg_b = re.compile(
    r"(android|bb\\d+|meego).+mobile|avantgo|bada\\/|blackberry|blazer|compal"
    r"|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|"
    r"mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|"
    r"re)\\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\\.(browser|"
    r"link)|vodafone|wap|windows ce|xda|xiino", re.I | re.M)
reg_v = re.compile(
    r"1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\\-)|"
    r"ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|"
    r"au(di|\\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|"
    r"bw\\-(n|u)|c55\\/|capi|ccwa|cdm\\-|cell|chtm|cldc|cmd\\-|co(mp|nd)|craw"
    r"|da(it|ll|ng)|dbte|dc\\-s|devi|dica|dmob|do(c|p)o|ds(12|\\-d)|el(49|ai)"
    r"|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\\-|_)|g1 u|g560"
    r"|gene|gf\\-5|g\\-mo|go(\\.w|od)|gr(ad|un)|haie|hcit|hd\\-(m|p|t)|"
    r"hei\\-|hi(pt|ta)|hp( i|ip)|hs\\-c|ht(c(\\-| |_|a|g|p|s|t)|tp)|"
    r"hu(aw|tc)|i\\-(20|go|ma)|i230|iac( |\\-|\\/)|ibro|idea|ig01|ikom|im1k"
    r"|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\\/)|klon|kpt |"
    r"kwc\\-|kyo(c|k)|le(no|xi)|lg( g|\\/(k|l|u)|50|54|\\-[a-w])|libw|lynx|"
    r"m1\\-w|m3ga|m50\\/|ma(te|ui|xo)|mc(01|21|ca)|m\\-cr|me(rc|ri)|mi(o8|oa|"
    r"ts)|mmef|mo(01|02|bi|de|do|t(\\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10"
    r"[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\\-|on|tf|wf|"
    r"wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|"
    r"pg(13|\\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\\-2|po(ck|rt|se)|prox|psio"
    r"|pt\\-g|qa\\-a|qc(07|12|21|32|60|\\-[2-7]|i\\-)|qtek|r380|r600|raks|rim9"
    r"|ro(ve|zo)|s55\\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\\-|oo|p\\-)|sdk\\/|se"
    r"(c(\\-|0|1)|47|mc|nd|ri)|sgh\\-|shar|sie(\\-|m)|sk\\-0|sl(45|id)|sm"
    r"(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\\-|v\\-|v )|sy(01|mb)|t2(18|50)|t6(00"
    r"|10|18)|ta(gt|lk)|tcl\\-|tdg\\-|tel(i|m)|tim\\-|t\\-mo|to(pl|sh)|ts(70"
    r"|m\\-|m3|m5)|tx\\-9|up(\\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40"
    r"|5[0-3]|\\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|"
    r"w3c(\\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\\-|your|zeto|"
    r"zte\\-", re.I | re.M)


def is_mobile(user_agent):
    """ Checks if the user browser from the given user agent is mobile.

    Args:
        user_agent: A given user agent.

    Returns: True if the browser from the user agent is mobile.

    """
    if user_agent:
        b = reg_b.search(user_agent)
        v = reg_v.search(user_agent[0:4])
        return b or v
    return False




from collections import namedtuple


def page_first_row(page, rows_per_page=10):
    return (int(page) * int(rows_per_page)) + 1 - int(rows_per_page)


class Paginator(object):

    def __init__(self, row_count, current_page=None, rows_per_page=10,
                 pages_per_block=10):
        if current_page is None or current_page == 0:
            current_page = 1

        self.row_count = row_count
        self.rows_per_page = rows_per_page
        self.rows_last_page = 0

        self.current_page = int(current_page)
        self.page_count = 1

        self.block_count = 0
        self.current_block = 0
        self.pages_per_block = pages_per_block
        self.pages_last_block = 0

        self.current_first_row = 0
        self.current_last_row = 0

        self.block_first_page = 0
        self.block_last_page = 0

        # Setting page count
        if self.row_count > 0:
            self.page_count = int(self.row_count/self.rows_per_page) + (
                1 if self.row_count % self.rows_per_page else 0)

        # Fixed current if overflows the row count
        if self.page_count < self.current_page:
            self.current_page = self.page_count

        # Setting rows in the last page
        self.rows_last_page = self.rows_per_page if not(
            self.row_count % self.rows_per_page) else (
            self.row_count % self.rows_per_page)

        # Setting block count
        self.block_count = int(self.page_count / self.pages_per_block) + (
            1 if self.page_count % self.pages_per_block else 0)

        # Setting pages in the last block of pages
        if self.page_count < self.pages_per_block:
            self.pages_last_block = self.page_count
        else:
            self.pages_last_block = self.pages_per_block if not(
                self.page_count % self.pages_per_block) else (
                self.page_count % self.pages_per_block)

        # Setting current block of pages
        if self.current_page <= self.pages_per_block:
            self.current_block = 1
        else:
            self.current_block = int(
                self.current_page / self.pages_per_block) + (
                1 if self.current_page % self.pages_per_block else 0)

        # Setting current first row
        self.current_first_row = page_first_row(self.current_page,
                                                self.rows_per_page)
        # Setting current last row
        if self.current_page == self.page_count:
            self.current_last_row = (self.current_first_row +
                                     self.rows_last_page - 1)
        else:
            self.current_last_row = (self.current_page * self.rows_per_page)

        # Setting current first page
        self.block_first_page = ((self.current_block * self.pages_per_block
                                    ) + 1 - self.pages_per_block)
        # Setting current last page
        if self.current_block == self.block_count:
            self.block_last_page = (self.block_first_page +
                                    self.pages_last_block - 1)
        else:
            self.block_last_page = (self.current_block *
                                    self.pages_per_block)

    @property
    def is_first_page(self):
        if self.current_page == 1:
            return True
        return False

    @property
    def is_last_page(self):
        if self.current_page == self.page_count:
            return True
        return False

    @property
    def block_pages(self):
        return range(self.block_first_page, self.block_last_page+1)

PagedData = namedtuple('PagedData', ['data', 'count'])



#!/usr/bin/env python
#
# Copyright 2015-2018 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division, print_function,
                        with_statement)

import tornado.web
import firenado.conf
from . import data
from . import session
from . import uimodules
from .config import get_class_from_config, load_yaml_config_file
import inspect
import logging
from tornado.escape import json_encode
from tornado.template import Loader
import tornado.websocket
import os
from six import iteritems, string_types


logger = logging.getLogger(__name__)


class TornadoApplication(tornado.web.Application, data.DataConnectedMixin,
                         session.SessionEnginedMixin):
    """Firenado basic Tornado application.
    """

    def __init__(self, default_host="", transforms=None, **settings):
        logger.debug('Wiring application located at %s.' %
                     firenado.conf.APP_ROOT_PATH)
        self.components = {}
        settings.update(firenado.conf.app['settings'])
        handlers = []
        ui_modules = []
        data.configure_data_sources(firenado.conf.app['data']['sources'], self)
        self.__load_components()
        for key, component in iteritems(self.components):
            component_handlers = component.get_handlers()
            for i in range(0, len(component_handlers)):
                if issubclass(
                    component_handlers[i][1], TornadoHandler
                ) or issubclass(
                    component_handlers[i][1], TornadoWebSocketHandler
                ):
                    if len(component_handlers[i]) < 3:
                        component_handlers[i] = (
                            component_handlers[i][0],
                            component_handlers[i][1],
                            {'component': component}
                        )
                    else:
                        component_handlers[i][1].component = component
            handlers = handlers + component_handlers
            # Adding component ui modules to the application ui modules list
            ui_modules.append(uimodules)
            if component.get_ui_modules():
                ui_modules.append(component.get_ui_modules())
        if firenado.conf.app['component']:
            if firenado.conf.app['static_path']:
                if os.path.isabs(firenado.conf.app['static_path']):
                    settings['static_path'] = firenado.conf.app['static_path']
                else:
                    settings['static_path'] = os.path.join(
                            self.components[firenado.conf.app[
                                'component']].get_component_path(),
                            firenado.conf.app['static_path'])
            else:
                settings['static_path'] = os.path.join(
                        self.components[
                            firenado.conf.app[
                                'component']].get_component_path(), 'static')
        else:
            settings['static_path'] = os.path.join(
                    os.path.dirname(__file__), "static")
        static_url_prefix = firenado.conf.app['static_url_prefix']
        if static_url_prefix != "/":
            static_url_prefix = "%s/" % static_url_prefix
        settings['static_url_prefix'] = static_url_prefix
        if len(ui_modules) > 0:
            settings['ui_modules'] = ui_modules
        if firenado.conf.app['cookie_secret']:
            settings['cookie_secret'] = firenado.conf.app['cookie_secret']
        if firenado.conf.app['xsrf_cookies']:
            settings['xsrf_cookies'] = firenado.conf.app['xsrf_cookies']
        if firenado.conf.app['url_root_path'] is not None:
            from .util.url_util import rooted_path
            for idx, handler in enumerate(handlers):
                handler_list = list(handler)
                handler_list[0] = rooted_path(
                    firenado.conf.app['url_root_path'], handler_list[0])
                handlers[idx] = tuple(handler_list)
        tornado.web.Application.__init__(self, handlers=handlers,
                                         default_host=default_host,
                                         transforms=transforms, **settings)
        logger.debug("Checking if session is enabled.")
        if firenado.conf.session['enabled']:
            logger.debug("Session is enabled. Starting session engine.")
            self.session_engine
        else:
            logger.debug("Session is disabled.")

    def get_app_component(self):
        return self.components[firenado.conf.app['component']]

    def __load_components(self):
        """ Loads all enabled components registered into the components
        conf.
        """
        for key, value in iteritems(firenado.conf.components):
            if value['enabled']:
                component_class = get_class_from_config(value)
                self.components[key] = component_class(key, self)
                if self.components[key].get_config_file():
                    from firenado.util.file import file_has_extension
                    filename = self.components[key].get_config_file()
                    comp_config_file = None
                    if file_has_extension(filename):
                        if os.path.isfile(os.path.join(
                                firenado.conf.APP_CONFIG_PATH, filename)):
                            comp_config_file = os.path.join(
                                firenado.conf.APP_CONFIG_PATH, filename)
                    else:
                        config_file_extensions = ['yml', 'yaml']
                        for extension in config_file_extensions:
                            candidate_filename = os.path.join(
                                    firenado.conf.APP_CONFIG_PATH,
                                    "%s.%s" % (filename, extension))
                            if os.path.isfile(candidate_filename):
                                comp_config_file = candidate_filename
                                break
                    if comp_config_file is not None:
                        self.components[key].conf = load_yaml_config_file(
                            comp_config_file)
                        self.components[key].process_config()
                    else:
                        logger.debug("Failed to find the file for the "
                                     "component %s at %s. Component filename "
                                     "returned is %s." % (
                                        key, firenado.conf.APP_CONFIG_PATH,
                                        self.components[key].get_config_file())
                                     )
        # Initializing enabled components after the load.
        for key, value in iteritems(firenado.conf.components):
            if value['enabled']:
                self.components[key].initialize()


class TornadoComponent(object):
    """ Firenado applications are organized in components. A component could be
    an application or something that can be distributed as an add-on or a
    plugin.
    """
    def __init__(self, name, application):
        self.name = name
        self.application = application
        self.conf = {}
        self.plugins = dict()

    def after_handler(self, handler):
        """ Add a logic to be executed after all component's handlers
        execution.
        """
        pass

    def before_handler(self, handler):
        """ Add a logic to be executed before all component's handler
        execution.
        """
        pass

    def get_handlers(self):
        """ Returns handlers being added by the component to the application.
        :return: A list of handlers the component provides.
        """
        return []

    def get_ui_modules(self):
        """ Returns uimodules the component provides to the application.
        It could be just a module, a list or a dictionary of modules.
        :return: Uimodules the component provides.
        """
        return None

    def get_component_path(self):
        """ Returns the component path.
        """
        return os.path.abspath(os.path.dirname(
            inspect.getfile(self.__class__)))

    def get_config_filename(self):
        return None

    def get_config_file(self):
        filename = self.get_config_filename()
        if filename is not None:
            return filename
        return None

    def get_template_path(self):
        """ Returns the path that holds the component's templates.
        """
        return os.path.join(os.path.abspath(os.path.dirname(
            inspect.getfile(self.__class__))), 'templates')

    def initialize(self):
        """ If you want to add logic while the component is initializing
        please overwrite this method.
        """
        pass

    def install(self):
        """ Firenado handles an application installation looping thought all
        components and triggering the install method of them.
        If
        """
        pass

    def process_config(self):
        """ To process your component configuration please overwrite this
        method reading the data on self.conf.
        """
        pass

    def shutdown(self):
        """ If you have resources that will hang after the shutdown please
        overwrite this method and close/unload those resources.
        """
        pass


class TornadoHandler(tornado.web.RequestHandler):
    """ Base request handler to be used on a Firenado application.
    It provides session and handles component paths.
    """
    def __init__(self, application, request, **kwargs):
        self.component = None
        self.session = None
        self.__template_variables = dict()
        super(TornadoHandler, self).__init__(application, request, **kwargs)

    def initialize(self, component):
        self.component = component

    def add_variable_to_template(self, name, variable):
        """ Add a variable to a dict that will be added to the template during
        the render or render_string execution.
        """
        self.__template_variables[name] = variable

    def authenticated(self):
        """ Returns if the current user is authenticated. If the current user
        is set then we consider authenticated.

        :return: bool True is current user is set
        """
        return self.current_user is not None;

    def is_mobile(self):
        from .util import browser
        if 'User-Agent' in self.request.headers:
            return browser.is_mobile(self.request.headers['User-Agent'])
        return False

    @session.read
    def prepare(self):
        self.component.before_handler(self)

    @session.write
    def on_finish(self):
        self.component.after_handler(self)

    def render_string(self, template_name, **kwargs):
        ignore_component = False
        application_component = None
        for key in ('ignore_component', 'component',):
            if key in kwargs:
                if key is 'ignore_component':
                    ignore_component = kwargs[key]
                if key is 'component':
                    pass
        kwargs['user_agent'] = self.user_agent if hasattr(
            self, 'user_agent') else None
        kwargs['credential'] = self.credential if hasattr(
            self, 'credential') else None
        for name, variable in iteritems(self.__template_variables):
            kwargs[name] = variable
        if self.ui:
            return super(TornadoHandler, self).render_string(
                template_name, **kwargs)
        else:
            # TODO: After a redirect I'm still hitting here.
            # Need to figure out what is going on.
            self._finished = False
            return None

    def write_error(self, status_code, **kwargs):
        error_stack = {'code': status_code}

        exc_info = None
        for key in kwargs:
            if key == 'exc_info':
                exc_info = kwargs[key]
        error = exc_info[1]

        if type(error) == JSONError:
            error_stack.update(error.data)
            response = dict(data=None, error=error_stack)
            self.write(response)
        else:
            raise error

    def get_data_connected(self):
        return self.application

    def get_firenado_template_path(self):
        """Override to customize the firenado template path for each handler.

        By default, we use the ``firenado_template_path`` application setting.
        Return None to load templates relative to the calling file.
        """
        return self.application.settings.get('firenado_template_path')

    def get_rooted_path(self, path):
        from .util.url_util import rooted_path
        root = firenado.conf.app['url_root_path']
        return rooted_path(root, path)

    def get_template_path(self):
        """Override to customize template path for each handler.

        By default, we use the ``template_path`` application setting.
        Return None to load templates relative to the calling file.
        """
        if self.component is None:
            # This is the default behaviour provided by Tornado.
            # No components on the request no fancy template path.
            return super(TornadoHandler, self).get_template_path()
        else:
            return self.component.get_template_path()

    def create_template_loader(self, template_path):
        """Returns a new template loader for the given path.

        May be overridden by subclasses.  By default returns a
        directory-based loader on the given path, using the
        ``autoescape`` application setting.  If a ``template_loader``
        application setting is supplied, uses that instead.
        """
        settings = self.application.settings
        kwargs = {}
        if 'autoescape' in settings:
            # autoescape=None means "no escaping", so we have to be sure
            # to only pass this kwarg if the user asked for it.
            kwargs['autoescape'] = settings['autoescape']
        return FirenadoComponentLoader(
            template_path, component=self.component, **kwargs)


class FirenadoComponentLoader(Loader):
    """ A template loader that loads from a single root directory.
    """
    def __init__(self, root_directory, component=None, **kwargs):
        # TODO: Check if we should alter/use the root_directory value
        # here or on the resolve_path method.
        self.component = component
        super(FirenadoComponentLoader, self).__init__(root_directory, **kwargs)

    def resolve_path(self, name, parent_path=None):
        """ When a template name comes with a ':' it means a template from
        another component is being referenced. The component template will be
        resolved and passed to the original Tornado resolve_path method.

        :param name: The template name
        :param parent_path: The template parent path
        :return: Tornado resolve_path result.
        """
        logger.debug("Resolving template %s." % name)
        name_resolved = name
        if ':' in name:
            nameX = name.split(':')
            component_name = nameX[0]
            name_resolved = os.path.join(
                self.component.application.components[
                    component_name].get_template_path(), nameX[-1])
        if name != name_resolved:
            logger.debug("Template %s resolved at %s." % (name, name_resolved))

        return super(FirenadoComponentLoader,
                     self).resolve_path(name_resolved, parent_path)


# TODO: We need to create a class to avoid those methods repeated here.
class TornadoWebSocketHandler(tornado.websocket.WebSocketHandler):

    def __init__(self, application, request, **kwargs):
        self.component = None
        self.__template_variables = dict()
        super(TornadoWebSocketHandler, self).__init__(application,
                                                      request, **kwargs)

    def initialize(self, component):
        self.component = component

    def add_variable_to_template(self, name, variable):
        """ Add a variable to a dict that will be added to the template during
        the render or render_string execution.
        """
        self.__template_variables[name] = variable

    @session.read
    def prepare(self):
        self.component.before_handler(self)

    @session.write
    def on_finish(self):
        self.component.after_handler(self)

    def render_string(self, template_name, **kwargs):
        ignore_component = False
        application_component = None
        for key in ('ignore_component', 'component',):
            if key in kwargs:
                if key is 'ignore_component':
                    ignore_component = kwargs[key]
                if key is 'component':
                    pass
        kwargs['user_agent'] = self.user_agent if hasattr(
            self, 'user_agent') else None
        kwargs['credential'] = self.credential if hasattr(
            self, 'credential') else None
        for name, variable in iteritems(self.__template_variables):
            kwargs[name] = variable
        if self.ui:
            return super(TornadoWebSocketHandler, self).render_string(
                template_name, **kwargs)
        else:
            # TODO: After a redirect I'm still hitting here.
            # Need to figure out what is going on.
            self._finished = False
            return None

    def write_error(self, status_code, **kwargs):
        error_stack = {'code': status_code}

        exc_info = None
        for key in kwargs:
            if key == 'exc_info':
                exc_info = kwargs[key]
        error = exc_info[1]

        if type(error) == JSONError:
            error_stack.update(error.data)
            response = dict(data=None, error=error_stack)
            self.write(response)
        else:
            raise error

    def get_firenado_template_path(self):
        """Override to customize the firenado template path for each handler.

        By default, we use the ``firenado_template_path`` application setting.
        Return None to load templates relative to the calling file.
        """
        return self.application.settings.get('firenado_template_path')

    def get_template_path(self):
        """Override to customize template path for each handler.

        By default, we use the ``template_path`` application setting.
        Return None to load templates relative to the calling file.
        """
        if self.component is None:
            # This is the default behaviour provided by Tornado.
            # No components on the request no fancy template path.
            return super(TornadoWebSocketHandler, self).get_template_path()
        else:
            return self.component.get_template_path()

    def create_template_loader(self, template_path):
        """Returns a new template loader for the given path.

        May be overridden by subclasses.  By default returns a
        directory-based loader on the given path, using the
        ``autoescape`` application setting.  If a ``template_loader``
        application setting is supplied, uses that instead.
        """
        settings = self.application.settings
        kwargs = {}
        if 'autoescape' in settings:
            # autoescape=None means "no escaping", so we have to be sure
            # to only pass this kwarg if the user asked for it.
            kwargs['autoescape'] = settings['autoescape']
        return FirenadoComponentLoader(
            template_path, component=self.component, **kwargs)


#TODO: This is iFlux migration leftover. Is that necessary?.
class JSONError(tornado.web.HTTPError):

    def __init__(self, status_code, log_message=None, *args, **kwargs):
        data = {}
        self.data.update(log_message)
        if not isinstance(log_message, string_types):
            json_log_message = self.data
            json_log_message['code'] = status_code
            json_log_message = json_encode(json_log_message)
        super(JSONError, self).__init__(
            status_code, json_log_message, *args, **kwargs)



#!/usr/bin/env python
#
# Copyright 2015-2018 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import firenado.conf
import logging
import os
import six
import sys
from six import iteritems
from tornado import gen

if six.PY3:
    try:
        import importlib
        reload = importlib.reload
    except AttributeError:
        # PY33
        import imp
        reload = imp.reload

logger = logging.getLogger(__name__)


class FirenadoLauncher(object):

    def __init__(self, addresses=None, dir=None, port=None, socket=None):
        self.addresses = addresses
        self.dir = dir
        self.port = port
        self.socket = socket

    def load(self):
        return None

    def launch(self):
        return None


class ProcessLauncher(FirenadoLauncher):
    import pexpect

    process = None  # type: pexpect.spawn

    def __init__(self, addresses=None, dir=None, port=None, socket=None,
                 logfile=None):
        super(ProcessLauncher, self).__init__(addresses, dir, port, socket)
        self.process = None
        self.process_callback = None
        self.logfile = logfile
        self.command = None
        self.response = None

    def load(self):
        sys.base_exec_prefix
        firenado_script = os.path.join(firenado.conf.ROOT, "bin",
                                       "firenado-cli.py")
        self.command = "%s %s app run" % (sys.executable, firenado_script)
        if self.dir is not None:
            dir_parameter = "--dir=%s" % self.dir
            self.command = "%s %s" % (self.command, dir_parameter)

        if self.socket is None:
            if self.addresses is not None:
                addresses_parameter = "--port=%s" % self.addresses
                self.command = "%s %s" % (self.command, addresses_parameter)
            if self.port is not None:
                port_parameter = "--port=%s" % self.port
                self.command = "%s %s" % (self.command, port_parameter)
        else:
            socket_parameter = "--port=%s" % self.socket
            self.command = "%s %s" % (self.command, socket_parameter)

    @gen.coroutine
    def read_process(self):
        import pexpect
        self.process_callback.stop()
        try:
            yield self.process.expect(r"[C|I|W|D|E].*", async_=True)
        except pexpect.TIMEOUT:
            pass
        self.process_callback.start()

    @gen.coroutine
    def launch(self):
        import pexpect
        import tornado.ioloop
        logger.info("Launching %s" % self.command)
        parameters = {
            'command': self.command,
            'encoding': "utf-8"
        }
        if self.logfile is not None:
            parameters['logfile'] = self.logfile
        self.process = pexpect.spawn(**parameters)
        yield self.process.expect("Firenado server started successfully.",
                                  async_=True)
        self.process_callback = tornado.ioloop.PeriodicCallback(
            self.read_process,
            400
        )
        self.process_callback.start()

    @gen.coroutine
    def shutdown(self):
        import pexpect
        yield self.process.expect(pexpect.EOF)


class TornadoLauncher(FirenadoLauncher):

    def __init__(self, addresses=None, dir=None, port=None, socket=None):
        super(TornadoLauncher, self).__init__(addresses, dir, port, socket)
        self.http_server = None
        self.application = None
        self.MAX_WAIT_SECONDS_BEFORE_SHUTDOWN = firenado.conf.app[
            'wait_before_shutdown']

    def load(self):
        from .tornadoweb import TornadoApplication
        if self.dir is not None:
            # TODO: This is a problem we cannot launch an app into the app
            os.chdir(self.dir)
            reload(firenado.conf)
        # TODO: Resolve module if doesn't exists
        if firenado.conf.app['pythonpath']:
            sys.path.append(firenado.conf.app['pythonpath'])
        self.application = TornadoApplication(debug=firenado.conf.app['debug'])

    def launch(self):
        import signal
        import tornado.httpserver
        signal.signal(signal.SIGTERM, self.sig_handler)
        signal.signal(signal.SIGINT, self.sig_handler)
        if os.name == "posix":
            signal.signal(signal.SIGTSTP, self.sig_handler)
        self.http_server = tornado.httpserver.HTTPServer(self.application)
        if firenado.conf.app['socket'] or self.socket:
            from tornado.netutil import bind_unix_socket
            socket_path = firenado.conf.app['socket']
            if self.socket:
                socket_path = self.socket
            socket = bind_unix_socket(socket_path)
            self.http_server.add_socket(socket)
            logger.info("Firenado listening at socket ""%s" %
                        socket.getsockname())
        else:
            addresses = self.addresses
            if addresses is None:
                addresses = firenado.conf.app['addresses']
            port = self.port
            if port is None:
                port = firenado.conf.app['port']
            for address in addresses:
                self.http_server.listen(port, address)
                logger.info("Firenado listening at ""http://%s:%s" % (address,
                                                                      port))
        logger.info("Firenado server started successfully.")
        tornado.ioloop.IOLoop.instance().start()

    def sig_handler(self, sig, frame):
        import tornado.ioloop
        logger.warning('Caught signal: %s', sig)
        tornado.ioloop.IOLoop.instance().add_callback(self.shutdown)

    def shutdown(self):
        import time
        import tornado.ioloop
        logger.info('Stopping http server')
        for key, component in iteritems(self.application.components):
            component.shutdown()
        self.http_server.stop()

        io_loop = tornado.ioloop.IOLoop.instance()

        if self.MAX_WAIT_SECONDS_BEFORE_SHUTDOWN == 0:
            io_loop.stop()
            logger.info('Application is down.')
        else:

            logger.info('Will shutdown in %s seconds ...',
                        self.MAX_WAIT_SECONDS_BEFORE_SHUTDOWN)
            deadline = time.time() + self.MAX_WAIT_SECONDS_BEFORE_SHUTDOWN

            def stop_loop():
                now = time.time()
                if now < deadline and (io_loop._callbacks or
                                       io_loop._timeouts):
                    io_loop.add_timeout(now + 1, stop_loop)
                else:
                    io_loop.stop()
                    logger.info('Application is down.')
            stop_loop()



#!/usr/bin/env python
#
# Copyright 2015-2018 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division, print_function,
                        with_statement)


import unittest
from firenado.util.file import file_has_extension, get_file_extension


class FileTestCase(unittest.TestCase):
    """ Case that tests the file util functions.
    """

    def test_extension_file_with_extension(self):
        """ Happy scenario, a filename with an extension
        """
        filename = 'the_file.ext'
        expected_extension = 'ext'
        extension = get_file_extension(filename)
        self.assertEqual(extension, expected_extension)
        self.assertTrue(file_has_extension(filename))

    def test_extension_file_ending_with_dot(self):
        """ Filename ending with a dot
        """
        filename = 'the_file.'
        extension = get_file_extension(filename)
        self.assertIsNone(extension)
        self.assertFalse(file_has_extension(filename))

    def test_extension_file_no_extension(self):
        """ Filename without extension
        """
        filename = 'the_file'
        extension = get_file_extension(filename)
        self.assertIsNone(extension)
        self.assertFalse(file_has_extension(filename))



#!/usr/bin/env python
#
# Copyright 2015-2018 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages
try:
    # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError:
    # for pip <= 9.0.3
    from pip.req import parse_requirements
import firenado
import os

# Solution from http://bit.ly/29Yl8VN
def resolve_requires(requirements_file):
    requirements = parse_requirements("./%s" % requirements_file,
            session=False)
    return [str(ir.req) for ir in requirements]

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
# Solution from: http://bit.ly/2mig8RT
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

# We still running: python setup.py sdist upload --repository=testpypi
# Twine isn't handling long_descriptions as per:
# https://github.com/pypa/twine/issues/262
setup(
    name="Firenado",
    version=".".join(map(str, firenado.__version__)),
    description="Firenado is a python web framework based on "
                "Tornado web framework/server.",
    long_description=read("README.rst"),
    license="Apache License V2.0",
    author="Flavio Garcia",
    author_email="piraz@candango.org",
    maintainer="Flavio Garcia",
    maintainer_email="piraz@candango.org",
    install_requires=resolve_requires("requirements/basic.txt"),
    extras_require={
        'redis': resolve_requires("requirements/redis.txt"),
        'pexpect': resolve_requires("requirements/pexpect.txt"),
        'sqlalchemy': resolve_requires("requirements/sqlalchemy.txt"),
    },
    url="https://github.com/candango/firenado",
    packages=find_packages(),
    package_dir={'firenado': "firenado"},
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Environment :: Web Environment",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    scripts=["firenado/bin/firenado-cli.py"],
    entry_points={'console_scripts': [
        "firenado = firenado.management:run_from_command_line",
    ]},
)
